/**
 * Definiert Rechtecke und stellt Methoden bereit, um deren Eigenschaften
 * und Beziehungen untereinander zu erfragen.
 * 
 * @author Owyn
 * @version 2008-05-01
 */
public class Rectangle {

	/**
	 * x-Koordinate der linken unteren Ecke
	 */
	int x;
	/**
	 * y-Koordinate der linken unteren Ecke
	 */
	int y;
	/**
	 * Breite
	 */
	int width;
	/**
	 * H&ouml;he
	 */
	int height;

	/**
	 * Initialisiert ein Rechteck.
	 *
	 * @param x x-Koordinate der linken unteren Ecke
	 * @param y y-Koordinate der linken unteren Ecke
	 * @param w Breite des Rechtecks
	 * @param h H&ouml;he des Rechtecks
	 */
	Rectangle(int x, int y, int w, int h) {
		this.x = x;
		this.y = y;
		this.width = w;
		this.height = h;
	}

	/**
	 * Gibt die Breite des Rechtecks zur&uuml;ck.
	 *
	 * @return Breite des Rechtecks.
	 */
	int getWidth() { return width; }

	/**
	 * Gibt die H&ouml;he des Rechtecks zur&uuml;ck.
	 *
	 * @return H&ouml;he des Rechtecks.
	 */
	int getHeight() { return height; }

	/**
	 * Gibt die x-Koordinate der linken Seite des Rechtecks zur&uuml;ck.
	 *
	 * @return x-Koordinate der linken Seite des Rechtecks
	 */
	int getLeft() { return x; }

	/**
	 * Gibt die x-Koordinate der rechten Seite des Rechtecks zur&uuml;ck.
	 * 
	 * @return x-Koordinate der rechten Seite des Rechtecks
	 */
	int getRight() { return x + width; }

	/**
	 * Gibt die y-Koordinate der unteren Seite des Rechtecks zur&uuml;ck.
	 * 
	 * @return y-Koordinate der unteren Seite des Rechtecks
	 */
	int getBottom() { return y; }

	/**
	 * Gibt die y-Koordinate der oberen Seite des Rechtecks zur&uuml;ck.
	 * 
	 * @return y-Koordinate der oberen Seite des Rechtecks
	 */
	int getTop() { return y + height; }

	/**
	 * Pr&uuml;ft, ob das Rechteck quadratisch ist.
	 * 
	 * @return true, falls das Rechteck quadratisch ist (sonst false).
	 */
	boolean isSquare() {
		//Ein Rechteck ist quadratisch, wenn Breite und Hoehe gleich sind.
		return width == height;
	}

	/**
	 * Pr&uuml;ft, ob das aktuelle Rechteck komplett in einem anderen liegt.
	 * 
	 * @param r zweites Rechteck
	 * @return true, wenn das aktuelle Rechteck komplett in r liegt.
	 */
	boolean isContained(Rectangle r) {
		return getLeft() >= r.getLeft()
			&& getBottom() >= r.getBottom()
			&& getRight() <= r.getRight()
			&& getTop() <= r.getTop();
	}

	/**
	 * Pr&uuml;ft, ob das aktuelle Rechteck keinen gemeinsamen Punkt mit einem anderen hat.
	 * 
	 * @param r zweites Rechteck
	 * @return true, wenn das aktuelle Rechteck und r keinen gemeinsamen Punkt haben.
	 */
	boolean isDisjoint(Rectangle r) {
		return getLeft() >= r.getRight()
			|| getBottom() >= r.getTop()
			|| getRight() <= r.getLeft()
			|| getTop() <= r.getBottom();
	}
	
	/**
	 * Gibt die Fl&auml;che des Rechtecks zur&uuml;ck.
	 * 
	 * @return Fl&auml;che des Rechtecks.
	 */
	int area() { return width * height; }
	
	/**
	 * Pr&uuml;ft, ob das aktuelle Rechteck einen gr&ouml;&szlig;eren Fl&auml;cheninhalt als ein anderes hat.
	 * 
	 * @param r zweites Rechteck
	 * @return true, wenn das aktuelle Rechteck gr&ouml;&szlig;er als r ist.
	 */
	boolean isGreater(Rectangle r) {
		return area() > r.area();
	}

}
